/* Simple NBTV monitor using Arduino Due

copyright 2018 Barry Chambers G8AGN

 v 0.1 3 Sept 2018  initial concept
 v 0.2 4 Sept 2018  use LM1881 sync separator to initiate line scan
 v 0.3 5 Sept 2018  corrected voltage associated with black (to account for waveforms where sync_low > 0 volts)
 v 0.4 6 Sept 2018  use NE555 to find missing sync pulse which denotes end of frame
 v 0.5 8 Sept 2018  changed mapBC function
 v 0.6 13 Sept 2018 changed to 2.2" 240x320 SPI TFT display
 v 0.7 14 Sept 2018 use new W x W super pixels to mimic (48 x W) by (32 x W) pixel display
*/

#include "SPI.h"
#include "Adafruit_GFX.h"
#include "Adafruit_ILI9341.h"

// SDA (MOSI) and SCL (CLK) on display connected to appropriate pins on Due SPI 6 pin connector next to the SAM3X8E CPU chip
#define tft_cs   24
#define tft_dc   26
#define tft_rst  22



int sig_white = 0;
int sync_low = 2000;
int sig_black;

// 8 red brightness levels
//uint16_t c[] = {0x1800, 0x3800, 0x5800, 0x7800, 0x9800, 0xB800, 0xD800, 0xF800};
// 16 red brightness levels
//uint16_t c[] = {0x0800, 0x1800, 0x2800, 0x3800, 0x4800, 0x5800, 0x6800, 0x7800, 0x8800, 0x9800, 0xA800, 0xB800, 0xC800, 0xD800, 0xE800, 0xF800};

// 8 green brightness levels
//uint16_t c[] = {0x00E0, 0x01E0, 0x02E0, 0x03E0, 0x04E0, 0x05E0, 0x06E0, 0x07E0};
// 16 green brightness levels
//uint16_t c[] = {0x0060, 0x00E0, 0x0160, 0x01E0, 0x0260, 0x02E0, 0x0360, 0x03E0, 0x0460, 0x04E0, 0x0560, 0x05E0, 0x0660, 0x06E0, 0x0760, 0x07E0};

// 8 blue brightness levels
//uint16_t c[] = {0x0003, 0x0007, 0x000B, 0x000F, 0x0013, 0x0017, 0x001B, 0x001F};
// 16 blue brightness levels
//uint16_t c[] = {0x0001, 0x0003, 0x0005, 0x0007, 0x0009, 0x000B, 0x000D, 0x000F, 0x0011, 0x 0013, 0x0015, 0x0017, 0x0019, 0x001B, 0x001D, 0x001F};

// 8 grey brightness levels
//uint16_t c[] = {0x18E3, 0x39E7, 0x5AEB, 0x7BEF, 0x9CF3, 0xBDF7, 0xDEFB, 0xFFFF};
//16 grey brightness levels
uint16_t c[] = {0x861, 0x18E3, 0x2965, 0x39E7, 0x4A69, 0x5AEB, 0x6B6D, 0x7BEF, 0x8C71, 0x9CF3, 0xAD75, 0xBDF7, 0xCE79, 0xDEFB, 0xEF7D, 0xFFFF};

// 8 colour bar (black, blue, red,magenta,green,cyan, yellow, white)
//uint16_t c[] ={0x0000, 0x001F, 0xF800, 0xF81F, 0x07E0, 0x7FF, 0xFFE0, 0xFFFF};

volatile int sig;       // video signal amplitude
volatile int  dot;      // pixel brightness

volatile uint16_t x = 159;    // x range is 159 (RHS) -> 32 (LHS) = 32 x W lines
volatile uint16_t y = 200;   // y range is 200 (bottom) -> 9 (top) = 48 x W pixels/line
uint16_t W = 4;              // width of super pixel

Adafruit_ILI9341 tft = Adafruit_ILI9341(tft_cs, tft_dc);


void setup() {

  Serial.begin(9600);
  tft.begin();
  tft.fillScreen(ILI9341_BLUE);
  yield();
  tft.setCursor(0,150);
  tft.setTextColor(ILI9341_YELLOW);
  tft.setTextSize(3);
  tft.setRotation(1);
  tft.println("   NBTV Monitor");
  delay(5000);
  tft.fillScreen(ILI9341_BLACK);
  yield();
  
  // modify ADC register to make it read faster
  //  change STARTUP from 8 to 2 (512 to 16 periods of ADC clock)
  REG_ADC_MR = (REG_ADC_MR & 0xFFF0FFFF) | 0x00020000;
  // change PRESCAL from 2 to 1
  REG_ADC_MR = (REG_ADC_MR & 0xFFFFF0FF) | 0x00000100;

  // find lowest voltage associated with the video sync pulses
  // find highest voltage associated with the video peak white
  for (int m = 0; m < 30000; m++)   // roughly 120ms total time
  {
    sig = analogRead(0);
    if (sig < sync_low)sync_low = sig;
    if (sig > sig_white)sig_white = sig;
   
  }

  // estimate black voltage level (typically 0.3 of peak white)
  sig_black = 0.3 * (float)(sig_white - sync_low) + sync_low;

  Serial.println(sync_low);
  Serial.println(sig_black);
  Serial.println(sig_white);
  
  attachInterrupt(4, line_scan, RISING);   // looking for  trailing edge of sync pulse going from LOW to HIGH
  attachInterrupt(5, new_frame, FALLING);   // NE555 provides missing sync pulse detection to reset frame
}

void line_scan()
{
  if (x > 31)   
  {
  for (y = 200; y > 8; y = y - 4)
  {
    delayMicroseconds(1);
    sig = analogRead(0);
    sig = constrain(sig,sig_black,sig_white);     

    dot = mapBC(sig, sig_black, sig_white, 0, 15);       // video amplitude mapped into 8 or 16 brightness levels depending on choice of c[]
    tft.drawFastVLine(x - W + 1, y, W, c[dot]); 

yield();
  }
  x = x - 4;
  }
}

void new_frame()
{
  x = 159;
  y = 200;  // reset coords to bottom RH corner of picture
}

long mapBC(long x, long in_min, long in_max, long out_min, long out_max)
{
  if ((in_max - in_min) > (out_max - out_min)) {
    return (x - in_min) * (out_max - out_min+1) / (in_max - in_min+1) + out_min;
  }
  else
  {
    return (x - in_min) * (out_max - out_min) / (in_max - in_min) + out_min;
  }
}


void loop ()
{
     while (1);
}

